/**
 * JSONEditor
 * component utilising Ajv JSON schema validator
 * https://www.npmjs.com/package/ajv
 */
import { useEffect, useMemo, useState } from "react";
import { Editor, EditorBox } from "./Editor";
import Ajv from "ajv";
import addFormats from "ajv-formats";
import jsonSourceMap from "json-source-map";
import { Alert } from "@mui/material";
import { useUniqueKeys } from "../../hooks/UseUniqueKeys";

const defaultOptions = {
  useWorker: false,
  displayIndentGuides: false,
  enableBasicAutocompletion: true,
  enableLiveAutocompletion: true,
  enableSnippets: false,
  showLineNumbers: true,
  tabSize: 2
};

const ajv = new Ajv({ allErrors: true });
addFormats(ajv);

// Dynamic Width (Build Regex)
// see https://stackoverflow.com/a/51506718
const wrapText = (s, w) =>
  s.replace(new RegExp(`(?![^\\n]{1,${w}}$)([^\\n]{1,${w}})\\s`, "g"), "$1\n");

function ajvErrorsToAnnotations(errors, pointers) {
  const annotations = errors.map((error) => {
    const { instancePath, message, params, keyword } = error;
    const type = "error";
    const width = 80;
    const custom = true;
    const messagePath = instancePath;
    if (keyword === "additionalProperties") {
      const { additionalProperty } = params;
      const combinedInstancePath = `${instancePath}/${additionalProperty}`;
      const { line: row, column } = pointers[combinedInstancePath].key;
      const text = `${messagePath} additional property "${additionalProperty}" not allowed`;
      return { row, column, type, text: wrapText(text, width), custom };
    } else if (keyword === "required") {
      const { missingProperty } = params;
      const { line: row, column } = pointers[instancePath].value;
      const text = `${messagePath} missing required property "${missingProperty}"`;
      return { row, column, type, text: wrapText(text, width), custom };
    } else if (keyword === "enum") {
      const { allowedValues } = params;
      const { line: row, column } = pointers[instancePath].value;
      const allowedValuesText = `allowedValues: ${JSON.stringify(
        allowedValues,
        null,
        2
      )}`;
      const text = `${messagePath} ${message}\n\n${allowedValuesText}`;
      return { row, column, type, text: wrapText(text, width), custom };
    } else {
      const { line: row, column } = pointers[instancePath].value;
      const text = `${messagePath} ${message}`;
      return { row, column, type, text: wrapText(text, width), custom };
    }
  });
  return annotations.sort((a, b) => a.row - b.row);
}

const JSONEditorDefaultOptions = {
  ...defaultOptions,
  useWorker: true // enable JSON syntax checking
};

function useValidatedJSON(json, schema) {
  const [wellFormed, setWellFormed] = useState(false);
  const [schemaAnnotations, setSchemaAnnotations] = useState([]);
  const validate = useMemo(() => ajv.compile(schema), [schema]);

  useEffect(() => {
    try {
      const { data, pointers } = jsonSourceMap.parse(json);
      setWellFormed(true);
      validate(data);
      setSchemaAnnotations(ajvErrorsToAnnotations(validate.errors, pointers));
    } catch (e) {
      setWellFormed(false);
      setSchemaAnnotations([]);
      console.debug(e);
    }
  }, [json, validate]);

  return { wellFormed, schemaAnnotations };
}

function AnnotationAlert({ key, annotation }) {
  const { row, text } = annotation;
  return (
    <Alert
      className="CeuiJSONEditorAlert"
      key={key}
      severity="error"
    >
      <pre style={{ margin: 0 }}>{`Error on line ${row + 1}: ${text}`}</pre>
    </Alert>
  );
}

export function JSONEditor({
  value,
  schema,
  onChange,
  options = JSONEditorDefaultOptions,
  showAlerts = true,
  maxAlerts = 3,
  className = "",
  ...delegated
}) {
  const { schemaAnnotations } = useValidatedJSON(value, schema);
  const [annotations, setAnnotations] = useState([]);
  const [editor, setEditor] = useState();
  const nextAnnotations = useMemo(
    () => [
      ...annotations.filter(({ custom }) => !custom), // annotations by worker (json syntax)
      ...schemaAnnotations // customa schema validation annotations
    ],
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [JSON.stringify(annotations), JSON.stringify(schemaAnnotations)]
  );
  const annotationsKeys = useUniqueKeys(annotations);

  useEffect(() => {
    editor?.getSession().setAnnotations(nextAnnotations);
  }, [editor, nextAnnotations]);

  return (
    <EditorBox className={`${className} CeuiJSONEditorBox`}>
      <Editor
        {...{
          onLoad: setEditor,
          onValidate: setAnnotations,
          mode: "json",
          height: "100%",
          value,
          schema,
          onChange,
          annotations,
          options,
          ...delegated
        }}
      />
      {showAlerts &&
        annotations.slice(0, maxAlerts).map((x, i) => (
          <AnnotationAlert
            key={annotationsKeys[i]}
            annotation={x}
          />
        ))}
    </EditorBox>
  );
}
