import { useCallback, useState } from "react";
import {
  Button,
  Dialog as MuiDialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  IconButton,
  styled,
  Box,
  Stack,
  Typography,
  Grid,
  TextField
} from "@mui/material";
import CloseIcon from "@mui/icons-material/Close";

/**
 * The title component for an ESS dialog, featuring a text/custom area and a close button
 * nestled inside a colored box.
 */
const DialogTitle = styled(({ onClose, className, children }) => {
  return (
    <Stack
      className={className}
      flexDirection="row"
      justifyContent="space-between"
      alignItems="center"
      backgroundColor="primary.main"
      color="essWhite.main"
      paddingX={2}
      paddingY={1}
    >
      {typeof children === "string" ? (
        <Typography>{children}</Typography>
      ) : (
        <>{children}</>
      )}
      <IconButton
        variant="outlined"
        onClick={onClose}
        aria-label="Close Dialog"
        edge="end"
      >
        <CloseIcon
          fontSize="medium"
          color="essWhite"
        />
      </IconButton>
    </Stack>
  );
})({});

/**
 * @name renderActionsCallback
 * @function
 * @param {function} onClose - callback fired when the component requests to be closed
 * @returns {object} JSX component
 */

/**
 * A generic MUI Dialog stylized for ESS that provides:
 *  - a colorful header (uses primary color) with a close button
 *  - a content area
 *  - an action/buttons area, with access to the onClose callback
 *
 * See:
 * https://mui.com/material-ui/react-dialog/
 * https://mui.com/material-ui/api/dialog/
 *
 * NOTE: If you need to provide a confirmation (yes/no) experience,
 * the ConfirmationDialog offers these features already.
 *
 * @param {string|object} title - Title string or custom component
 * @param {string|object} content - Content string or custom component
 * @param {renderActionsCallback} renderActions - render function that receives the onClose callback and returns the dialog footer/actions
 * @param {boolean} open return true or false
 * @param {function} onClose callback fired when the component requests to be closed
 * @param {object} DialogProps MUI Dialog component props; this might be how you change the
 * width of the modal via fullWidth=true, maxWidth="md", for example. See MUI documentation.
 * @param {string} className containing css classname
 */
export const Dialog = ({
  title,
  content,
  renderActions,
  open,
  onClose,
  DialogProps = {},
  className
}) => {
  return (
    <MuiDialog
      open={open}
      onClose={onClose}
      fullWidth
      maxWidth="sm"
      {...DialogProps}
    >
      {title ? <DialogTitle onClose={onClose}>{title}</DialogTitle> : null}
      <DialogContent className={className}>
        {typeof content === "string" ? (
          <DialogContentText>{content}</DialogContentText>
        ) : (
          <Box>{content}</Box>
        )}
      </DialogContent>
      {renderActions ? (
        <DialogActions>{renderActions(onClose)}</DialogActions>
      ) : null}
    </MuiDialog>
  );
};

const ConfirmationDialogButtons = styled(
  ({
    onClose,
    onConfirm,
    onCancel,
    confirmText,
    cancelText,
    isConfirmDisabled = false
  }) => {
    const handleConfirm = useCallback(() => {
      onClose();
      if (onConfirm) {
        onConfirm();
      }
    }, [onClose, onConfirm]);

    const handleCancel = useCallback(() => {
      onClose();
      if (onCancel) {
        onCancel();
      }
    }, [onCancel, onClose]);

    return (
      <>
        <Button
          className="CeuiConfirmationDialog-CancelButton"
          onClick={handleCancel}
          color="primary"
        >
          {cancelText}
        </Button>
        <Button
          className="CeuiConfirmationDialog-ConfirmButton"
          onClick={handleConfirm}
          color="primary"
          variant="contained"
          autoFocus
          disabled={isConfirmDisabled}
        >
          {confirmText}
        </Button>
      </>
    );
  }
)({});

/**
 * An extended ESS Dialog that provides cancel and confirm buttons. Clicking either
 * buttons fires onCancel or onConfirm callbacks, and automatically closes the Dialog.
 *
 * NOTE: If you need to control the closing behavior, for example to display a form and
 * only close the dialog once a server request completes, consider extending Dialog
 * on your own.
 *
 * ConfirmationDialog is meant only for simple confirmation from the user!
 *
 * @param {function} onConfirm callback confirming whether confirm button was clicked
 * @param {function} onCancel callback confirming whether cancel button was clicked
 * @param {string} confirmText text for confirm button
 * @param {string} cancelText text for cancel button
 * @param {object} ...props - Props passed to the underlying Dialog component (ours, not MUIs).
 * @param {string|object} props.title - Title string or custom component
 * @param {string|object} props.content - Content string or custom component
 * @param {renderActionsCallback} props.renderActions - render function that receives the onClose callback and returns the dialog footer/actions
 * @param {boolean} props.open return true or false
 * @param {function} props.onClose callback fired when the component requests to be closed
 * @param {object} props.DialogProps MUI Dialog component props; this might be how you change the
 * width of the modal via fullWidth=true, maxWidth="md", for example. See MUI documentation.
 * @param {string} props.className containing css classname
 */
export const ConfirmationDialog = styled(
  ({
    onConfirm,
    onCancel,
    confirmText = "Yes",
    cancelText = "No",
    isConfirmDisabled,
    ...props
  }) => {
    return (
      <Dialog
        renderActions={(onClose) => (
          <ConfirmationDialogButtons
            onClose={onClose}
            onConfirm={onConfirm}
            onCancel={onCancel}
            cancelText={cancelText}
            confirmText={confirmText}
            isConfirmDisabled={isConfirmDisabled}
          />
        )}
        {...props}
      />
    );
  }
)({});

export const ConfirmDangerActionDialog = ({ ...props }) => {
  const { title, text, confirmText, valueToCheck, open, onClose, onConfirm } =
    props;
  const [isConfirmDisabled, setIsConfirmDisabled] = useState(true);

  const handleInputChange = (event) => {
    const inputText = event.target.value;
    setIsConfirmDisabled(inputText !== valueToCheck);
  };

  const handleClose = () => {
    setIsConfirmDisabled(true);
    onClose();
  };

  return (
    <ConfirmationDialog
      title={title}
      confirmText={confirmText}
      cancelText="Cancel"
      onClose={handleClose}
      content={
        <Grid
          container
          spacing={2}
        >
          <Grid
            item
            xs={12}
          >
            {typeof content === "string" ? (
              <DialogContentText>{text}</DialogContentText>
            ) : (
              <Box>{text}</Box>
            )}
          </Grid>
          <Grid
            item
            xs={12}
          >
            <TextField
              autoComplete="off"
              variant="outlined"
              label={valueToCheck}
              fullWidth
              onChange={handleInputChange}
            />
          </Grid>
        </Grid>
      }
      {...{ open, onConfirm, isConfirmDisabled }}
    />
  );
};
